#pragma once

#include <SDL2/SDL_ttf.h>

#include "texture.h"
#include "surface.h"

#define FONT_GLYPH_MAX 128

#define STRING_SDL_TTF_FONT_SUCCESS "[INFO] Loaded font: %s\n"
#define STRING_SDL_TTF_FONT_ERROR "[ERROR] Unable to load font: %s | %s\n"

typedef struct Font
{
	TTF_Font* ttf;
	Texture glyphTextures[FONT_GLYPH_MAX];
	u32 size;
	bool isInit;
} Font;

typedef struct GlyphMetrics
{
	s32 minX;
	s32 minY;
	s32 maxX;
	s32 maxY;
	s32 advance;
} GlyphMetrics;

bool font_init(Font* self, const char* path, u32 size);
s32 font_glyph_kerning_get(Font* self, char glyph, char nextGlyph);
s32 font_glyph_width_get(Font* self, char glyph);
s32 font_height_get(Font* self);
s32 font_line_skip_get(Font* self);
void font_free(Font* self);
void font_glyph_metrics_init(Font* self, GlyphMetrics* glyphMetrics, char glyph);
void font_glyph_texture_init(Font* self, Texture* texture, char glyph);

static const SDL_Color GLYPH_COLOR = {255, 255, 255, 255};
