#pragma once

#include "component/COMPONENT_COMMON.h"

#include "../../engine/vector.h"
#include "../../engine/renderer.h"
#include "../../engine/random.h"

#define ECS_FUNCTION_COUNT ECS_FUNCTION_DRAW + 1
typedef enum ECSFunctionType
{
	ECS_FUNCTION_ADD,
	ECS_FUNCTION_DELETE,
	ECS_FUNCTION_TICK,
	ECS_FUNCTION_UPDATE,
	ECS_FUNCTION_DRAW
} ECSFunctionType;

typedef struct ECS ECS;
typedef struct Control Control;
typedef struct Input Input;
typedef struct Renderer Renderer;
typedef struct Postprocessing Postprocessing;
typedef struct Resources Resources;
typedef u32 EntityID;

typedef void (*ECSFunction)(void*, ECS*);

typedef struct ECSSystem
{
	ECSFunction functions[ECS_FUNCTION_COUNT];
} ECSSystem;

typedef struct Component
{
	EntityID id;
	bool isDisabled;
	bool isFunctionDisabled[ECS_FUNCTION_COUNT];
} Component;

typedef struct ComponentInfo
{
	ECSSystem system;
	ComponentType type;
	size_t size;
} ComponentInfo;

typedef struct ComponentList
{
	ECS* ecs;
	ECSSystem system;
	ComponentType type;
	Vector components; 
} ComponentList;

typedef struct ECS
{
	Input* input;
	Control* control;
	Renderer* renderer;
	Resources* resources;
	Postprocessing* postprocessing;
	ComponentList lists[COMPONENT_COUNT];
	bool isFunctionDisabled[ECS_FUNCTION_COUNT];
	bool isLog;
	EntityID nextID;
} ECS;

