#include "component_animation_player.h"

void _component_animation_player_set(ComponentAnimationPlayer* self, ECS* ecs, AnimationPlayerType type);
void _component_animation_player_determine(ComponentAnimationPlayer* self, ECS* ecs);

void
_component_animation_player_set(ComponentAnimationPlayer* self, ECS* ecs, AnimationPlayerType type)
{
	ComponentAnimation* animation;

	if (type == self->type)
		return;

	self->type = type;

	animation = ecs_component_get(ecs, COMPONENT_ANIMATION, self->component.id);

	component_animation_set(animation, ecs, &ANIMATION_PLAYER_STRUCTS[type]);
}

void
_component_animation_player_determine(ComponentAnimationPlayer* self, ECS* ecs)
{
	AnimationPlayerType type;
	ComponentWorldObject* worldObject;

	worldObject = ecs_component_get(ecs, COMPONENT_WORLD_OBJECT, self->component.id);

	type = (AnimationPlayerType)(self->action + ((ANIMATION_PLAYER_ACTION_COUNT) * worldObject->direction));

	_component_animation_player_set(self, ecs, type);
}

void
component_animation_player_init(ComponentAnimationPlayer* self, ECS* ecs, AnimationPlayerType type)
{
	self->type = -1;
	self->action = ANIMATION_PLAYER_ACTION_IDLE;

	_component_animation_player_set(self, ecs, type);
}

void
component_animation_player_tick(ComponentAnimationPlayer* self, ECS* ecs)
{
	ComponentWorldObject* worldObject;
	ComponentPlayerControl* playerControl;
	ComponentPhysics* physics;
	ComponentAnimation* animation;
	ComponentTimer* animationTimer;
	f32 nextPositionDistance;
	s32 newTimer;

	worldObject = ecs_component_get(ecs, COMPONENT_WORLD_OBJECT, self->component.id);
	
	if (worldObject->isAirborne && self->action == ANIMATION_PLAYER_ACTION_LUNGE)
		return;

	playerControl = ecs_component_get(ecs, COMPONENT_PLAYER_CONTROL, self->component.id);
	physics = ecs_component_get(ecs, COMPONENT_PHYSICS, self->component.id);

	self->action = ANIMATION_PLAYER_ACTION_IDLE;

	if (playerControl->isLunge)
	{
		self->action = ANIMATION_PLAYER_ACTION_LUNGE;
		sound_play(&ecs->resources->sounds[SOUND_LUNGE + random_s32_in_range(0, SOUND_LUNGE_COUNT - 1)]);
	}
	else if (physics->isMoving)
	{
		self->action = ANIMATION_PLAYER_ACTION_RUN;

		animation = ecs_component_get(ecs, COMPONENT_ANIMATION, self->component.id);
		animationTimer = ecs_component_get(ecs, COMPONENT_TIMER, animation->timer);
		
		nextPositionDistance = component_physics_next_position_distance_get(physics, ecs);

		newTimer = ANIMATION_PLAYER_RUN_TIMER - (s32)((nextPositionDistance / ANIMATION_PLAYER_RUN_DISTANCE_MAX) * ANIMATION_PLAYER_RUN_TIMER);

		newTimer = MIN(newTimer, (s32)ANIMATION_PLAYER_RUN_TIMER_MIN);

		if (animationTimer->value > (s32)ANIMATION_PLAYER_RUN_TIMER_MIN)
			animationTimer->value = (s32)(CLAMP((s32)animationTimer->value, (s32)ANIMATION_PLAYER_RUN_TIMER_MIN, (s32)newTimer));
	}

	_component_animation_player_determine(self, ecs);
}
