#pragma once

#include "component_homing.h"
#include "component_collect.h"
#include "component_world_object.h"
#include "component_levitate.h"
#include "component_food.h"
#include "component_power.h"
#include "component_stamina.h"

#include "../visual/component_color_change.h"

static const f32 COMPONENT_COLLECTIBLE_HEIGHT_OFFSET = 1000.0f;
static const f32 COMPONENT_COLLECTIBLE_RADIUS = 64.0f;
static const u32 COMPONENT_COLLECTIBLE_FOOD_VALUE = 1;
static const u32 COMPONENT_COLLECTIBLE_ROTTEN_FOOD_VALUE = -5;
static const f32 COMPONENT_COLLECTIBLE_POWER_VALUE = 10.0f;
static const f32 COMPONENT_COLLECTIBLE_STAMINA_VALUE = 50.0f;
static const u32 COMPONENT_COLLECTIBLE_HOMING_SPEED = 3.0f;

static const f32 COMPONENT_COLLECTIBLE_LEVITATE_DISTANCE = 32.0f;
static const f32 COMPONENT_COLLECTIBLE_LEVITATE_FREQUENCY = 10.0f;

static const u32 COMPONENT_COLLECTIBLE_STAMINA_COLOR_CHANGE_TIME = 30;
static const u32 COMPONENT_COLLECTIBLE_POWER_COLOR_CHANGE_TIME = 30;

static const vec4 COMPONENT_COLLECTIBLE_POWER_COLOR = {0.60f, 1.0f, 0.80f, 0.0f};
static const vec4 COMPONENT_COLLECTIBLE_STAMINA_COLOR = {1.0f, 0.2f, 0.40f, 0.0f};

typedef struct Vector Vector;

#define COLLECTIBLE_TYPE_COUNT COLLECTIBLE_ROTTEN_FOOD + 1
typedef enum CollectibleType
{
	COLLECTIBLE_FOOD,
	COLLECTIBLE_STAMINA,
	COLLECTIBLE_POWER,
	COLLECTIBLE_LOREBOOK,
	COLLECTIBLE_AMULET,
	COLLECTIBLE_ROTTEN_FOOD
} CollectibleType;

typedef struct ComponentCollectible
{
	Component component;
	CollectibleType type;
} ComponentCollectible;

void
component_collectible_init
(
	ComponentCollectible* self, 
	ECS* ecs,
	Texture* texture, 
	const vec2 size, 
	const vec3 position,
	CollectibleType type
);

void component_collectible_add(ComponentCollectible* self, ECS* ecs);
void component_collectible_tick(ComponentCollectible* self, ECS* ecs);

#define COMPONENT_COLLECTIBLE_DEPENDENCY_COUNT 3
static const ComponentType COMPONENT_COLLECTIBLE_DEPENDENCIES[COMPONENT_COLLECTIBLE_DEPENDENCY_COUNT] = 
{
	COMPONENT_WORLD_OBJECT,
	COMPONENT_HOMING,
	COMPONENT_LEVITATE
};

static const ComponentInfo COMPONENT_COLLECTIBLE_INFO = 
{
	.system = 
	{
		.functions = 
		{ 
			(ECSFunction)component_collectible_add, 
			NULL,
			(ECSFunction)component_collectible_tick, 
			NULL,
			NULL
		}
	},
	.type = COMPONENT_COLLECTIBLE,
	.size = sizeof(ComponentCollectible)
};
