#include "component_plane_object.h"

void _component_plane_object_depth_set(ComponentPlaneObject* self, ECS* ecs);
void _component_plane_object_circle_collider_set(ComponentPlaneObject* self, ECS* ecs);

void
_component_plane_object_circle_collider_set(ComponentPlaneObject* self, ECS* ecs)
{
	ComponentCircleCollider* circleCollider;
	ComponentTextureQuad* textureQuad;
	ComponentPhysics* physics;

	physics = ecs_component_get(ecs, COMPONENT_PHYSICS, self->component.id);
	textureQuad = ecs_component_get(ecs, COMPONENT_TEXTURE_QUAD, self->component.id);
	circleCollider = ecs_component_get(ecs, COMPONENT_CIRCLE_COLLIDER, self->component.id);

	circleCollider->circle.position[0] = physics->position[0];
	circleCollider->circle.position[1] = physics->position[1] + (textureQuad->size[1] / 2.0f);
}

void
_component_plane_object_depth_set(ComponentPlaneObject* self, ECS* ecs)
{
	ComponentPhysics* physics;
	ComponentTextureQuad* textureQuad;
	f32 depth;
	f32 yOffset;

	physics = ecs_component_get(ecs, COMPONENT_PHYSICS, self->component.id);
	textureQuad = ecs_component_get(ecs, COMPONENT_TEXTURE_QUAD, self->component.id);

	yOffset = COMPONENT_PLANE_OBJECT_DEPTH_Y_OFFSET_MULTIPLIER * ((physics->position[1] + textureQuad->size[1]) / COMPONENT_PLANE_OBJECT_BOUNDS[3]);

	depth = COMPONENT_PLANE_OBJECT_DEPTH_MAX - yOffset;
	depth += COMPONENT_PLANE_OBJECT_DEPTH_MIN;

	physics->position[2] = depth;
}

void
_component_plane_object_bounds_set(ComponentPlaneObject* self, ECS* ecs)
{
	ComponentPhysics* physics;

	physics = ecs_component_get(ecs, COMPONENT_PHYSICS, self->component.id);

	physics->position[0] = CLAMP(physics->position[0], COMPONENT_PLANE_OBJECT_BOUNDS[0], COMPONENT_PLANE_OBJECT_BOUNDS[2]);
	physics->position[1] = CLAMP(physics->position[1], COMPONENT_PLANE_OBJECT_BOUNDS[1], COMPONENT_PLANE_OBJECT_BOUNDS[3]);
}


void
component_plane_object_init
(
	ComponentPlaneObject* self, 
	ECS* ecs,
	Texture* texture, 
	vec2 size, 
	vec3 position,
	f32 radius
)
{
	component_game_object_init
	(
		ecs_component_get(ecs, COMPONENT_GAME_OBJECT, self->component.id),
		ecs, 
		texture, 
		COMPONENT_PLANE_OBJECT_BUFFER,
		COMPONENT_PLANE_OBJECT_ORIGIN,
		size, 
		position
	);

	component_circle_collider_init
	(
		ecs_component_get(ecs, COMPONENT_CIRCLE_COLLIDER, self->component.id),
		(f32*)position,
		radius
	);

	component_physics_init
	(
		ecs_component_get(ecs, COMPONENT_PHYSICS, self->component.id),
		ecs,
		position,
		COMPONENT_PLANE_OBJECT_VELOCITY_MAX,
		COMPONENT_PLANE_OBJECT_FRICTION
	);

	_component_plane_object_circle_collider_set(self, ecs);
}

void
component_plane_object_add(ComponentPlaneObject* self, ECS* ecs)
{
	ecs_components_add
	(
		ecs, 
		COMPONENT_PLANE_OBJECT_DEPENDENCIES, 
		COMPONENT_PLANE_OBJECT_DEPENDENCY_COUNT, 
		self->component.id
	);
}

void
component_plane_object_tick(ComponentPlaneObject* self, ECS* ecs)
{
	_component_plane_object_bounds_set(self, ecs);
	_component_plane_object_depth_set(self, ecs);
	_component_plane_object_circle_collider_set(self, ecs);
}
