#pragma once

#include "component_game_object.h"
#include "../physics/component_circle_collider.h"

#include "../../entity/visual/entity_sprite.h"

static const RendererBuffer COMPONENT_WORLD_OBJECT_BUFFER = RENDERER_BUFFER_WORLD;
static const f32 COMPONENT_WORLD_OBJECT_SHADOW_Z_OFFSET = 0.001f;
static const f32 COMPONENT_WORLD_OBJECT_HEIGHT_OFFSET_SHADOW_SCALE_MULTIPLIER = 0.005f;
static const f32 COMPONENT_WORLD_OBJECT_FRICTION = 0.985f;
static const f32 COMPONENT_WORLD_OBJECT_SHADOW_HEIGHT = 32.0f;
static const f32 COMPONENT_WORLD_OBJECT_DEPTH_Y_OFFSET_MULTIPLIER = 0.1f;
static const f32 COMPONENT_WORLD_OBJECT_DEPTH_MIN = 0.1f;
static const f32 COMPONENT_WORLD_OBJECT_DEPTH_MAX = 0.2f;
static const f32 COMPONENT_WORLD_OBJECT_GRAVITY = 1.0f;
static const vec3 COMPONENT_WORLD_OBJECT_VELOCITY_MAX = {100.0f, 100.0f, 100.0f};
static const vec4 COMPONENT_WORLD_OBJECT_BOUNDS = {250.0f, 250.0f, 6150.0f, 3200.0f};

#define COMPONENT_WORLD_OBJECT_ORIGIN ORIGIN_CENTER

typedef struct ComponentWorldObject
{
	Component component;
	EntityID shadowID;
	f32 heightOffset;
	f32 heightOffsetVelocity;
	f32 angle;
	Direction direction;
	bool isAirborne;
	bool isGravityAffected;
	bool isShadow;
} ComponentWorldObject;

void
component_world_object_init
(
	ComponentWorldObject* self, 
	ECS* ecs,
	Texture* texture, 
	vec2 size, 
	vec3 position,
	f32 heightOffset,
	f32 radius,
	bool isShadow
);

void component_world_object_add(ComponentWorldObject* self, ECS* ecs);
void component_world_object_delete(ComponentWorldObject* self, ECS* ecs);
void component_world_object_tick(ComponentWorldObject* self, ECS* ecs);

#define COMPONENT_WORLD_OBJECT_DEPENDENCY_COUNT 2
static const ComponentType COMPONENT_WORLD_OBJECT_DEPENDENCIES[COMPONENT_WORLD_OBJECT_DEPENDENCY_COUNT] = 
{
	COMPONENT_GAME_OBJECT,
	COMPONENT_CIRCLE_COLLIDER
};

static const ComponentInfo COMPONENT_WORLD_OBJECT_INFO = 
{
	.system = 
	{
		.functions = 
		{ 
			(ECSFunction)component_world_object_add, 
			(ECSFunction)component_world_object_delete, 
			(ECSFunction)component_world_object_tick, 
			NULL,
			NULL
		}
	},
	.type = COMPONENT_WORLD_OBJECT,
	.size = sizeof(ComponentWorldObject)
};
