#include "component_physics.h"

void
component_physics_tick(ComponentPhysics* self, ECS* ecs)
{
	glm_vec3_add(self->velocity, self->position, self->position);

	glm_vec3_scale(self->velocity, self->friction, self->velocity);

	self->velocity[0] = MAX(self->velocity[0], self->velocityMax[0]);
	self->velocity[1] = MAX(self->velocity[1], self->velocityMax[1]);
	self->velocity[2] = MAX(self->velocity[2], self->velocityMax[2]);

	if 
	(
	 	self->velocity[0] != 0.0f ||
		self->velocity[1] != 0.0f
	)
		self->isMoving = true;
	else
		self->isMoving = false;
}

void
component_physics_next_position_get(ComponentPhysics* self, ECS* ecs, vec2 position)
{
	position[0] = self->position[0] + self->velocity[0];
	position[1] = self->position[1] + self->velocity[1];
}

f32
component_physics_next_position_distance_get(ComponentPhysics* self, ECS* ecs)
{
	vec2 position;

	component_physics_next_position_get(self, ecs, position);

	return DISTANCE_2D
	(
		position[0],
		self->position[0],
		position[1],
		self->position[1]
	);
}

void
component_physics_add(ComponentPhysics* self, ECS* ecs)
{
	self->friction = COMPONENT_PHYSICS_FRICTION_DEFAULT;
	glm_vec3_copy((f32*)COMPONENT_PHYSICS_VELOCITY_MAX_DEFAULT, self->velocityMax);
}

void
component_physics_init
(
 	ComponentPhysics* self,
	ECS* ecs,
	const vec3 position,
	const vec3 velocityMax,
	f32 friction
)
{
	glm_vec3_copy((f32*)position, self->position);
	glm_vec3_copy((f32*)velocityMax, self->velocityMax);
	self->friction = friction;
}
