#include "component_dialogue.h"

static void _component_dialogue_string_set(ComponentDialogue* self, ECS* ecs);

static void
_component_dialogue_string_set(ComponentDialogue* self, ECS* ecs)
{
	ComponentText* text;
	char string[COMPONENT_TEXT_STRING_MAX];

	memset(string, '\0', COMPONENT_TEXT_STRING_MAX);
	memcpy(string, self->string, self->index + 1);

	text = ecs_component_get(ecs, COMPONENT_TEXT, self->component.id);

	component_text_string_set(text, ecs, string, self->index + 1);

	sound_play(&ecs->resources->sounds[SOUND_BLIP]);
}

void
component_dialogue_add(ComponentDialogue* self, ECS* ecs)
{
	for (s32 i = 0 ; i < COMPONENT_DIALOGUE_DEPENDENCY_COUNT; i++)
		ecs_component_add(ecs, COMPONENT_DIALOGUE_DEPENDENCIES[i], self->component.id);

	self->timerID = entity_timer_add(ecs, COMPONENT_DIALOGUE_SPEED_DEFAULT);
	self->skipTimerID = entity_timer_add(ecs, COMPONENT_DIALOGUE_SKIP_TIMER_VALUE);
}

void
component_dialogue_delete(ComponentDialogue* self, ECS* ecs)
{
	for (s32 i = 0 ; i < COMPONENT_DIALOGUE_DEPENDENCY_COUNT; i++)
		ecs_component_delete(ecs, COMPONENT_DIALOGUE_DEPENDENCIES[i], self->component.id);

	ecs_entity_delete(ecs, self->timerID);
	ecs_entity_delete(ecs, self->skipTimerID);
}

void
component_dialogue_tick(ComponentDialogue* self, ECS* ecs)
{
	ComponentTimer* timer;

	if (self->isFinished)
		return;

	timer = ecs_component_get(ecs, COMPONENT_TIMER, self->timerID);

	if (timer->isFinished)
	{
		self->index++;

		_component_dialogue_string_set(self, ecs);

		if (self->index >= self->length)
			self->isFinished = true;
		else
			component_timer_init(timer, ecs, self->speed);
	}

	if (control_released(ecs->control, CONTROL_ACTION_PRIMARY))
	{
		ComponentTimer* skipTimer;
	
		skipTimer = ecs_component_get(ecs, COMPONENT_TIMER, self->skipTimerID);
	
		if (!skipTimer->isFinished)
			return;

		self->index = self->length - 1;
		timer->value = 0;
		timer->isFinished = true;

		component_timer_init(timer, ecs, self->speed);
		component_timer_init(skipTimer, ecs, COMPONENT_DIALOGUE_SKIP_TIMER_VALUE);
	}
}

void
component_dialogue_init
(
	ComponentDialogue* self, 
	ECS* ecs,
	const char* string,
	u32 speed
)
{
	ComponentTimer* timer;
	ComponentTimer* skipTimer;

	self->speed = speed;

	memcpy(self->string, string, COMPONENT_TEXT_STRING_MAX);

	timer = ecs_component_get(ecs, COMPONENT_TIMER, self->timerID);
	skipTimer = ecs_component_get(ecs, COMPONENT_TIMER, self->skipTimerID);

	component_timer_init(timer, ecs, self->speed);
	component_timer_init(skipTimer, ecs, COMPONENT_DIALOGUE_SKIP_TIMER_VALUE);

	self->length = strlen(self->string);
	self->index = 0;
	self->isFinished = false;

	_component_dialogue_string_set(self, ecs);
}
