#include "component_texture_quad.h"

void
component_texture_quad_size_get(ComponentTextureQuad* self, vec2 size)
{
	glm_vec2_copy(self->size, size);
	glm_vec2_mul(size, self->scale, size);
}

void
component_texture_quad_position_get(ComponentTextureQuad* self, vec3 position)
{
	vec2 size;

	glm_vec3_copy(self->position, position);
	glm_vec3_add(position, self->offset, position);

	component_texture_quad_size_get(self, size);

	switch (self->origin)
	{
		case ORIGIN_CENTER:
			position[0] -= size[0] / 2;
			position[1] -= size[1] / 2;
			break;
		case ORIGIN_TOP_LEFT:
		default:
			break;
	}

}

void
component_texture_quad_origin_point_get(ComponentTextureQuad* self, vec3 originPoint)
{
	vec2 size;

	glm_vec3_zero(originPoint);

	switch (self->origin)
	{
		case ORIGIN_CENTER:
			component_texture_quad_size_get(self, size);
			originPoint[0] = size[0] / 2;
			originPoint[1] = size[1] / 2;
		case ORIGIN_TOP_LEFT:
		default:
			break;
	}
}

void
component_texture_quad_model_get(ComponentTextureQuad* self, mat4 model)
{
	vec3 position;
	vec3 originPoint;

	component_texture_quad_position_get(self, position);
	component_texture_quad_origin_point_get(self, originPoint);

	glm_mat4_identity(model);
	glm_translate(model, position);
	glm_rotate_at(model, originPoint, self->rotation, (f32*)COMPONENT_TEXTURE_QUAD_ROTATION_AXIS);
}

void
component_texture_quad_rectangle_get(ComponentTextureQuad* self, Rectangle* rectangle)
{
	vec2 size;
	vec2 difference;

	component_texture_quad_size_get(self, size);

	rectangle->x = (self->position[0] - (size[0] / 2)) + self->offset[0];
	rectangle->y = (self->position[1] - (size[1] / 2)) + self->offset[1];
	rectangle->w = size[0];
	rectangle->h = size[1];
}

s32
component_texture_quad_sort(ComponentTextureQuad* a, ComponentTextureQuad* b)
{
	return a->position[2] > b->position[2] ? -1 : 1;
}

void 
component_texture_quad_init
(
	ComponentTextureQuad* self, 
	ECS* ecs, 
	Texture* texture,
	Flip flip,
	Origin origin,
	RendererBuffer buffer,
	f32 rotation,
	vec2 size, 
	vec2 scale,
	vec2 uvMin, 
	vec2 uvMax, 
	vec3 offset,
	vec3 position, 
	vec4 color
)
{

	self->texture = texture;
	self->flip = flip;
	self->origin = origin;
	self->buffer = buffer;
	self->rotation = rotation;
	glm_vec2_copy(size, self->size);
	glm_vec2_copy(scale, self->scale);
	glm_vec2_copy(uvMin, self->uvMin);
	glm_vec2_copy(uvMax, self->uvMax);
	glm_vec3_copy(COMPONENT_TEXTURE_QUAD_OFFSET_DEFAULT, self->offset);
	glm_vec4_copy(COMPONENT_TEXTURE_QUAD_COLOR_COVER, self->colorCover);
	glm_vec3_copy(position, self->position);
	glm_vec4_copy(color, self->color);
}

void
component_texture_quad_add(ComponentTextureQuad* self, ECS* ecs)
{
	component_texture_quad_init
	(
		self,
		ecs,
		&ecs->resources->textures[COMPONENT_TEXTURE_QUAD_TEXTURE_DEFAULT],
		COMPONENT_TEXTURE_QUAD_FLIP_DEFAULT,
		COMPONENT_TEXTURE_QUAD_ORIGIN_DEFAULT,
		COMPONENT_TEXTURE_QUAD_BUFFER_DEFAULT,
		COMPONENT_TEXTURE_QUAD_ROTATION_DEFAULT,
		COMPONENT_TEXTURE_QUAD_SIZE_DEFAULT,
		COMPONENT_TEXTURE_QUAD_SCALE_DEFAULT,
		COMPONENT_TEXTURE_QUAD_UV_MIN_DEFAULT,
		COMPONENT_TEXTURE_QUAD_UV_MAX_DEFAULT,
		COMPONENT_TEXTURE_QUAD_OFFSET_DEFAULT,
		COMPONENT_TEXTURE_QUAD_POSITION_DEFAULT,
		COMPONENT_TEXTURE_QUAD_COLOR_DEFAULT
	);
}

void
component_texture_quad_draw(ComponentTextureQuad* self, ECS* ecs)
{
	mat4 model;
	mat4 view;
	mat4 projection;
	vec2 size;

	component_texture_quad_model_get(self, model);
	component_texture_quad_size_get(self, size);

	camera_view_get(&ecs->renderer->camera[self->buffer], view);
	camera_projection_get(&ecs->renderer->camera[self->buffer], projection);

	renderer_buffer_use(ecs->renderer, self->buffer);

	texture_quad_draw
	(
	 	self->texture,
		ecs->renderer,
		&ecs->resources->shaders[SHADER_TEXTURE_QUAD],
		self->uvMin,
		self->uvMax,
		model,
		view,
		projection,
		size,
		self->color,
		self->colorCover,
		self->flip
	);

	renderer_buffer_unbind();
}
