#include "ecs_component_list.h"

void
ecs_component_list_init(ComponentList* self, ECS* ecs, ComponentInfo info)
{
	self->ecs = ecs;
	self->type = info.type;
	self->system = info.system;

	vector_init(&self->components, info.size);
}

void
ecs_component_list_function(ComponentList* self, ECSFunctionType type)
{
	ECSFunction function;

	function = self->system.functions[type];

	if (!function)
		return;

	for (s32 i = 0; i < (s32)self->components.count; i++)
	{
		void* component;
		Component ecsComponent;

		component = vector_get(&self->components, i);

		memcpy(&ecsComponent, component, sizeof(Component));

		if
		(
			ecsComponent.isDisabled ||
			ecsComponent.isFunctionDisabled[type]
		)
			continue;

		function(component, self->ecs);
	}
}

void*
ecs_component_list_get(ComponentList* self, EntityID id)
{
	for (s32 i = 0; i < (s32)self->components.count; i++)
	{
		u32 checkID;
		void* component;

		component = vector_get(&self->components, i);

		memcpy(&checkID, component, sizeof(u32));

		if (checkID == id)
			return component;
	}

	return NULL;
}

void
ecs_component_list_clear(ComponentList* self)
{
	vector_clear(&self->components);
}

void
ecs_component_list_free(ComponentList* self)
{
	vector_free(&self->components);
	memset(self, '\0', sizeof(ComponentList));
}
