#pragma once

#include "../../engine/file.h"
#include "../../engine/shader.h"
#include "../../engine/font.h"
#include "../../engine/texture.h"
#include "../../engine/sound.h"
#include "../../engine/music.h"

#define SHADER_COUNT SHADER_TEXTURE_QUAD + 1
typedef enum ShaderType
{
	SHADER_TEXTURE_QUAD
} ShaderType;

typedef struct ShaderPaths
{
	const char* vertex;
	const char* fragment;
} ShaderPaths;

static const ShaderPaths SHADER_PATHS[SHADER_COUNT] =
{
	{
		.vertex = "res/shader/texture_quad.vs",
		.fragment = "res/shader/texture_quad.fs"
	}
};

#define TEXTURE_COUNT TEXTURE_ENDING_THREE + 1
typedef enum TextureType
{
	TEXTURE_DEFAULT,
	TEXTURE_CURSOR,
	TEXTURE_LOGO,
	TEXTURE_TITLE_BG,
	TEXTURE_LEVEL_SELECT_BUTTONS,
	TEXTURE_LEVEL_SELECT_ICONS,
	TEXTURE_METER,
	TEXTURE_METER_STAMINA,
	TEXTURE_METER_POWER,
	TEXTURE_MEDALS,
	TEXTURE_GAME_BG,
	TEXTURE_GAME_PLANE,
	TEXTURE_SHADOW,
	TEXTURE_PLAYER,
	TEXTURE_TARGET,
	TEXTURE_COLLECTIBLE,
	TEXTURE_SMOKE,
	TEXTURE_TRAP,
	TEXTURE_FOLIAGE,
	TEXTURE_CUTSCENE_ONE_SCENE_ONE,
	TEXTURE_CUTSCENE_ONE_SCENE_TWO,
	TEXTURE_CUTSCENE_ONE_SCENE_THREE,
	TEXTURE_CUTSCENE_ONE_SCENE_FOUR,
	TEXTURE_CUTSCENE_TWO_SCENE_ONE,
	TEXTURE_CUTSCENE_TWO_SCENE_TWO,
	TEXTURE_CUTSCENE_TWO_SCENE_THREE,
	TEXTURE_CUTSCENE_TWO_SCENE_FOUR,
	TEXTURE_CUTSCENE_TWO_SCENE_FIVE,
	TEXTURE_CUTSCENE_THREE_SCENE_ONE,
	TEXTURE_CUTSCENE_THREE_SCENE_TWO,
	TEXTURE_CUTSCENE_THREE_SCENE_THREE,
	TEXTURE_CUTSCENE_THREE_SCENE_FOUR,
	TEXTURE_CUTSCENE_THREE_SCENE_FIVE,
	TEXTURE_CUTSCENE_THREE_SCENE_SIX,
	TEXTURE_CUTSCENE_FOUR_SCENE_ONE,
	TEXTURE_CUTSCENE_FOUR_SCENE_TWO,
	TEXTURE_CUTSCENE_FOUR_SCENE_THREE,
	TEXTURE_CUTSCENE_FOUR_SCENE_FOUR,
	TEXTURE_CUTSCENE_FOUR_SCENE_FIVE,
	TEXTURE_CUTSCENE_FOUR_SCENE_SIX,
	TEXTURE_CUTSCENE_FIVE_SCENE_ONE,
	TEXTURE_CUTSCENE_FIVE_SCENE_TWO,
	TEXTURE_CUTSCENE_FIVE_SCENE_THREE,
	TEXTURE_CUTSCENE_FIVE_SCENE_FOUR,
	TEXTURE_CUTSCENE_FIVE_SCENE_FIVE,
	TEXTURE_CUTSCENE_FIVE_SCENE_SIX,
	TEXTURE_CUTSCENE_FINAL_SCENE_ONE,
	TEXTURE_CUTSCENE_FINAL_SCENE_TWO,
	TEXTURE_CUTSCENE_FINAL_SCENE_THREE,
	TEXTURE_CUTSCENE_FINAL_SCENE_FOUR,
	TEXTURE_CUTSCENE_FINAL_SCENE_FIVE,
	TEXTURE_CUTSCENE_FINAL_SCENE_SIX,
	TEXTURE_CUTSCENE_FINAL_SCENE_SEVEN,
	TEXTURE_CUTSCENE_FINAL_SCENE_EIGHT,
	TEXTURE_CUTSCENE_FINAL_SCENE_NINE,
	TEXTURE_CUTSCENE_FINAL_SCENE_TEN,
	TEXTURE_CUTSCENE_FINAL_SCENE_ELEVEN,
	TEXTURE_CUTSCENE_FINAL_SCENE_TWELVE,
	TEXTURE_CUTSCENE_FINAL_SCENE_THIRTEEN,
	TEXTURE_ENDING,
	TEXTURE_ENDING_TWO,
	TEXTURE_ENDING_THREE
} TextureType;

static const char* TEXTURE_PATHS[TEXTURE_COUNT] =
{
	"res/gfx/default.png",
	"res/gfx/cursor.png",
	"res/gfx/logo.png",
	"res/gfx/titleBG.png",
	"res/gfx/levelSelectButtons.png",
	"res/gfx/levelSelectIcons.png",
	"res/gfx/meter.png",
	"res/gfx/meterStamina.png",
	"res/gfx/meterPower.png",
	"res/gfx/medals.png",
	"res/gfx/gameBG.png",
	"res/gfx/gamePlane.png",
	"res/gfx/shadow.png",
	"res/gfx/player.png",
	"res/gfx/target.png",
	"res/gfx/collectible.png",
	"res/gfx/smoke.png",
	"res/gfx/trap.png",
	"res/gfx/foliage.png",
	"res/gfx/cutscene/c01s01.png",
	"res/gfx/cutscene/c01s02.png",
	"res/gfx/cutscene/c01s03.png",
	"res/gfx/cutscene/c01s04.png",
	"res/gfx/cutscene/c02s01.png",
	"res/gfx/cutscene/c02s02.png",
	"res/gfx/cutscene/c02s03.png",
	"res/gfx/cutscene/c02s04.png",
	"res/gfx/cutscene/c02s05.png",
	"res/gfx/cutscene/c03s01.png",
	"res/gfx/cutscene/c03s02.png",
	"res/gfx/cutscene/c03s03.png",
	"res/gfx/cutscene/c03s04.png",
	"res/gfx/cutscene/c03s05.png",
	"res/gfx/cutscene/c03s06.png",
	"res/gfx/cutscene/c04s01.png",
	"res/gfx/cutscene/c04s02.png",
	"res/gfx/cutscene/c04s03.png",
	"res/gfx/cutscene/c04s04.png",
	"res/gfx/cutscene/c04s05.png",
	"res/gfx/cutscene/c04s06.png",
	"res/gfx/cutscene/c05s01.png",
	"res/gfx/cutscene/c05s02.png",
	"res/gfx/cutscene/c05s03.png",
	"res/gfx/cutscene/c05s04.png",
	"res/gfx/cutscene/c05s05.png",
	"res/gfx/cutscene/c05s06.png",
	"res/gfx/cutscene/c06s01.png",
	"res/gfx/cutscene/c06s02.png",
	"res/gfx/cutscene/c06s03.png",
	"res/gfx/cutscene/c06s04.png",
	"res/gfx/cutscene/c06s05.png",
	"res/gfx/cutscene/c06s06.png",
	"res/gfx/cutscene/c06s07.png",
	"res/gfx/cutscene/c06s08.png",
	"res/gfx/cutscene/c06s09.png",
	"res/gfx/cutscene/c06s10.png",
	"res/gfx/cutscene/c06s11.png",
	"res/gfx/cutscene/c06s12.png",
	"res/gfx/cutscene/c06s13.png",
	"res/gfx/cutscene/ending.png",
	"res/gfx/cutscene/ending2.png",
	"res/gfx/cutscene/ending3.png"
};

#define FONT_COUNT FONT_BIG + 1
typedef enum FontType
{
	FONT_DEFAULT,
	FONT_SMALL,
	FONT_BIG
} FontType;

static const char* FONT_PATHS[FONT_COUNT] =
{
	"res/font/default.ttf",
	"res/font/default.ttf",
	"res/font/default.ttf"
};

static const u32 FONT_SIZES[FONT_COUNT] =
{
	32,
	24,
	48
};

#define SOUND_COUNT (SOUND_CANCEL + 1)
//#define SOUND_COUNT (SOUND_CHEAT + 1)
#define SOUND_BURP_COUNT 3
#define SOUND_GURGLE_COUNT 3
#define SOUND_LUNGE_COUNT 3
#define SOUND_GULP_COUNT 3
typedef enum SoundType
{
	SOUND_BLIP,
	SOUND_SELECT,
	SOUND_PAUSE,
	SOUND_UNPAUSE,
	SOUND_BRONZE,
	SOUND_SILVER,
	SOUND_GOLD,
	SOUND_EVENT,
	SOUND_LOW_TIMER_TICK,
	SOUND_FINISH,
	SOUND_LUNGE,
	SOUND_LUNGE_2,
	SOUND_LUNGE_3,
	SOUND_GULP,
	SOUND_GULP_2,
	SOUND_GULP_3,
	SOUND_BURP,
	SOUND_BURP_2,
	SOUND_BURP_3,
	SOUND_GURGLE,
	SOUND_GURGLE_2,
	SOUND_GURGLE_3,
	SOUND_APPEAR,
	SOUND_STICKY,
	SOUND_FOOD_COLLECT,
	SOUND_CANT_CARRY,
	SOUND_BLEH,
	SOUND_POWER,
	SOUND_STAMINA,
	SOUND_MAGNET,
	SOUND_EXTEND,
	SOUND_FORESIGHT,
	SOUND_INVIGORATE,
	SOUND_CANCEL//,
	//SOUND_CHEAT
} SoundType;

static const char* SOUND_PATHS[SOUND_COUNT] =
{
	"res/sound/blip.ogg",
	"res/sound/select.ogg",
	"res/sound/pause.ogg",
	"res/sound/unpause.ogg",
	"res/sound/bronze.ogg",
	"res/sound/silver.ogg",
	"res/sound/gold.ogg",
	"res/sound/event.ogg",
	"res/sound/lowTimerTick.ogg",
	"res/sound/finish.ogg",
	"res/sound/lunge.ogg",
	"res/sound/lunge2.ogg",
	"res/sound/lunge3.ogg",
	"res/sound/gulp.ogg",
	"res/sound/gulp2.ogg",
	"res/sound/gulp3.ogg",
	"res/sound/burp.ogg",
	"res/sound/burp2.ogg",
	"res/sound/burp3.ogg",
	"res/sound/gurgle.ogg",
	"res/sound/gurgle2.ogg",
	"res/sound/gurgle3.ogg",
	"res/sound/appear.ogg",
	"res/sound/sticky.ogg",
	"res/sound/foodCollect.ogg",
	"res/sound/cantCarry.ogg",
	"res/sound/bleh.ogg",
	"res/sound/power.ogg",
	"res/sound/stamina.ogg",
	"res/sound/magnet.ogg",
	"res/sound/extend.ogg",
	"res/sound/foresight.ogg",
	"res/sound/invigorate.ogg",
	"res/sound/cancel.ogg"//,
	//"res/sound/cheat.ogg"
};

#define MUSIC_COUNT MUSIC_VICTORY + 1
typedef enum MusicType
{
	MUSIC_TITLE,
	MUSIC_CUTSCENE,
	MUSIC_WIND,
	MUSIC_GAME,
	MUSIC_VICTORY
} MusicType;

static const char* MUSIC_PATHS[MUSIC_COUNT] =
{
	"res/music/title.ogg",
	"res/music/cutscene.ogg",
	"res/music/wind.ogg",
	"res/music/game.ogg",
	"res/music/victory.ogg"
};

typedef struct Resources
{
	Shader shaders[SHADER_COUNT];
	Texture textures[TEXTURE_COUNT];
	Music music[MUSIC_COUNT];
	Sound sounds[SOUND_COUNT]; 
	Font fonts[FONT_COUNT];
} Resources;
