#include "ending.h"

void
ending_init(Ending* self, ECS* ecs, EndingType type)
{
	Texture* texture;
	const char* tipString;

	memset(self, '\0', sizeof(Ending));

	self->ecs = ecs;

	self->cursor = entity_cursor_add(self->ecs);

	self->text = entity_text_add
	(
		ecs,
		&ecs->resources->fonts[FONT_BIG], 
		RENDERER_BUFFER_UI,
		STRING_ENDING_TEXT,
		ENDING_TEXT_POSITION,
		-1
	);


	switch (type)
	{
		case ENDING_ONE:
		default:
			texture = &ecs->resources->textures[TEXTURE_ENDING];
			break;
		case ENDING_TWO:
			texture = &ecs->resources->textures[TEXTURE_ENDING_TWO];
			break;
		case ENDING_THREE:
			texture = &ecs->resources->textures[TEXTURE_ENDING_THREE];
			break;
	};


	tipString = STRING_ENDING_TIP_TEXT[type];

	self->tipText = entity_text_add
	(
		ecs,
		&ecs->resources->fonts[FONT_SMALL], 
		RENDERER_BUFFER_UI,
		tipString,
		ENDING_TIP_TEXT_POSITION,
		-1
	);

	self->bg = entity_sprite_add
	(
		ecs, 
		texture,
		RENDERER_BUFFER_BACKGROUND, 
		ORIGIN_TOP_LEFT,
		ENDING_BG_SIZE,
		ENDING_BG_POSITION
	);

	music_play(&self->ecs->resources->music[MUSIC_WIND], true);
}

void
ending_tick(Ending* self)
{
	if (self->isFinished)
		return;

	if (control_released(self->ecs->control, CONTROL_ACTION_PRIMARY))
		self->isFinished = true;
}

void
ending_free(Ending* self)
{

}

