#include "ui.h"

void _level_ui_threshold_tick(Level* self);

void
level_ui_event_timer_init(Level* self)
{
	if (self->eventTimerUI != 0)
		ecs_entity_delete(self->ecs, self->eventTimerUI);

	self->eventTimerUI = entity_value_text_add
	(
		self->ecs, 
		&self->ecs->resources->fonts[FONT_SMALL],
		LEVEL_EVENT_TIMER_BUFFER, 
		LEVEL_EVENT_TIMER_POSITION, 
		LEVEL_EVENT_TIMER_COMPONENT_TYPE,
		LEVEL_EVENT_TIMER_VALUE_TEXT_TYPE,
		LEVEL_EVENT_TIMER_FORMAT, 
		self->eventTimer
	);
}

void
_level_ui_threshold_tick(Level* self)
{
	ComponentFood* food;

	if (self->medal == LEVEL_MEDAL_GOLD)
		return;

	food = ecs_component_get(self->ecs, COMPONENT_FOOD, self->target);

	for (s32 i = 0; i < (LEVEL_MEDAL_COUNT - 1); i++)
	{
		u32 thresholdValue;
		u32 medal;

		medal = (LEVEL_MEDAL_COUNT - 1) - i;

		thresholdValue = self->settings.medalThresholds[medal - 1];

		if (food->value >= (s32)thresholdValue)
		{
			ComponentAtlas* medalAtlas;
			ComponentThreshold* threshold;

			self->medal = medal;

			if (self->nextMedal <= self->medal)
			{
				switch(self->medal)
				{
					case (LEVEL_MEDAL_GOLD):
						level_text_queue_add(self, STRING_LEVEL_GOLD);
						sound_play(&self->ecs->resources->sounds[SOUND_GOLD]);
						break;
					case (LEVEL_MEDAL_SILVER):
						level_text_queue_add(self, STRING_LEVEL_SILVER);
						sound_play(&self->ecs->resources->sounds[SOUND_SILVER]);
						break;
					case (LEVEL_MEDAL_BRONZE):
						level_text_queue_add(self, STRING_LEVEL_BRONZE);
						sound_play(&self->ecs->resources->sounds[SOUND_BRONZE]);
						break;
					default:
						break;
				}

				self->timeToMedal = self->settings.time - self->timerValue; 
			}
			else
				return;

			self->nextMedal = self->medal + 1;

			self->nextMedal = MAX(self->nextMedal, LEVEL_MEDAL_GOLD);

			medalAtlas = ecs_component_get(self->ecs, COMPONENT_ATLAS, self->medalUI);
			threshold = ecs_component_get(self->ecs, COMPONENT_THRESHOLD, self->target);

			threshold->value = self->settings.medalThresholds[self->nextMedal - 1];
			medalAtlas->index = self->nextMedal - 1;

			break;
		}
	}
}

void
level_ui_init(Level* self)
{
	self->cursor = entity_cursor_add(self->ecs);

	self->playerFoodUI = entity_value_text_add
	(
		self->ecs, 
		&self->ecs->resources->fonts[FONT_DEFAULT],
		LEVEL_PLAYER_FOOD_UI_BUFFER, 
		LEVEL_PLAYER_FOOD_UI_POSITION, 
		LEVEL_PLAYER_FOOD_UI_COMPONENT_TYPE,
		LEVEL_PLAYER_FOOD_UI_VALUE_TEXT_TYPE,
		LEVEL_PLAYER_FOOD_UI_FORMAT, 
		self->player
	);

	self->targetFoodUI = entity_value_text_add
	(
		self->ecs, 
		&self->ecs->resources->fonts[FONT_DEFAULT],
		LEVEL_TARGET_FOOD_UI_BUFFER, 
		LEVEL_TARGET_FOOD_UI_POSITION, 
		LEVEL_TARGET_FOOD_UI_COMPONENT_TYPE,
		LEVEL_TARGET_FOOD_UI_VALUE_TEXT_TYPE,
		LEVEL_TARGET_FOOD_UI_FORMAT, 
		self->target
	);

	self->timerUI = entity_value_text_add
	(
		self->ecs, 
		&self->ecs->resources->fonts[FONT_DEFAULT],
		LEVEL_TIMER_BUFFER, 
		LEVEL_TIMER_POSITION, 
		LEVEL_TIMER_COMPONENT_TYPE,
		LEVEL_TIMER_VALUE_TEXT_TYPE,
		LEVEL_TIMER_FORMAT, 
		self->timer
	);

	self->thresholdUI = entity_value_text_add
	(
		self->ecs, 
		&self->ecs->resources->fonts[FONT_SMALL],
		LEVEL_THRESHOLD_UI_BUFFER, 
		LEVEL_THRESHOLD_UI_POSITION, 
		LEVEL_THRESHOLD_UI_COMPONENT_TYPE,
		LEVEL_THRESHOLD_UI_VALUE_TEXT_TYPE,
		LEVEL_THRESHOLD_UI_FORMAT, 
		self->target
	);

	self->medalUI = entity_medal_add
	(
		self->ecs, 
		LEVEL_MEDAL_POSITION,
		0
	);

	self->meterStamina = entity_meter_add
	(
		self->ecs, 
		&self->ecs->resources->textures[TEXTURE_METER_STAMINA], 
		LEVEL_PLAYER_METER_STAMINA_POSITION,
		COMPONENT_STAMINA,
		self->player
	);

	if (self->settings.isPower)
	{
		self->meterPower = entity_meter_add
		(
			self->ecs, 
			&self->ecs->resources->textures[TEXTURE_METER_POWER], 
			LEVEL_PLAYER_METER_POWER_POSITION,
			COMPONENT_POWER,
			self->player
		);
	}

	if (self->settings.isEvent)
		level_ui_event_timer_init(self);
}

void
level_ui_tick(Level* self)
{
	ComponentFood* food;

	_level_ui_threshold_tick(self);

	food = ecs_component_get(self->ecs, COMPONENT_FOOD, self->player);

	if (food->value >= food->max)
	{
		ComponentText* text;

		text = ecs_component_get(self->ecs, COMPONENT_TEXT, self->playerFoodUI);

		glm_vec4_copy(LEVEL_PLAYER_FOOD_UI_MAX_COLOR, text->color);
	}
	else
	{
		ComponentText* text;

		text = ecs_component_get(self->ecs, COMPONENT_TEXT, self->playerFoodUI);

		glm_vec4_copy(OPAQUE, text->color);
	}

}
